from typing import Any

from sensor_state_data import (
    BinarySensorDeviceClass,
    DeviceClass,
    DeviceKey,
    Event,
    SensorData,
    SensorDescription,
    SensorDeviceClass,
    SensorLibrary,
    SensorUpdate,
    SensorValue,
    Units,
)


def test_sensor_device_class():
    assert DeviceClass.TEMPERATURE == "temperature"
    assert str(DeviceClass.TEMPERATURE) == "temperature"  # type: ignore[unreachable]


def test_binary_sensor_device_class():
    assert BinarySensorDeviceClass.POWER == "power"
    assert str(BinarySensorDeviceClass.POWER) == "power"  # type: ignore[unreachable]


def test_no_precision():
    class MySensorData(SensorData):
        def _start_update(self, data: Any) -> None:
            self.update_predefined_sensor(
                SensorLibrary.TEMPERATURE__CELSIUS, 30.3232039
            )

    data = MySensorData()

    update = data.update(b"")
    assert update == SensorUpdate(
        title=None,
        devices={},
        entity_descriptions={
            DeviceKey(key="temperature", device_id=None): SensorDescription(
                device_key=DeviceKey(key="temperature", device_id=None),
                device_class=SensorDeviceClass.TEMPERATURE,
                native_unit_of_measurement=Units.TEMP_CELSIUS,
            )
        },
        entity_values={
            DeviceKey(key="temperature", device_id=None): SensorValue(
                device_key=DeviceKey(key="temperature", device_id=None),
                name="Temperature",
                native_value=30.3232039,
            )
        },
        binary_entity_descriptions={},
        binary_entity_values={},
        events={},
    )


def test_with_precision():
    class MySensorData(SensorData):
        def _start_update(self, data: Any) -> None:
            self.set_precision(2)
            self.update_predefined_sensor(
                SensorLibrary.TEMPERATURE__CELSIUS, 30.3232039
            )

    data = MySensorData()

    update = data.update(b"")
    assert update == SensorUpdate(
        title=None,
        devices={},
        entity_descriptions={
            DeviceKey(key="temperature", device_id=None): SensorDescription(
                device_key=DeviceKey(key="temperature", device_id=None),
                device_class=SensorDeviceClass.TEMPERATURE,
                native_unit_of_measurement=Units.TEMP_CELSIUS,
            )
        },
        entity_values={
            DeviceKey(key="temperature", device_id=None): SensorValue(
                device_key=DeviceKey(key="temperature", device_id=None),
                name="Temperature",
                native_value=30.32,
            )
        },
        binary_entity_descriptions={},
        binary_entity_values={},
        events={},
    )


def test_with_precision_between_updates():
    class MySensorData(SensorData):
        def _start_update(self, data: Any) -> None:
            self.set_precision(2)
            self.update_predefined_sensor(
                SensorLibrary.TEMPERATURE__CELSIUS, 30.3232039
            )
            self.set_precision(4)
            self.update_predefined_sensor(
                SensorLibrary.TEMPERATURE__CELSIUS, 30.3232039
            )

    data = MySensorData()

    update = data.update(b"")
    assert update == SensorUpdate(
        title=None,
        devices={},
        entity_descriptions={
            DeviceKey(key="temperature", device_id=None): SensorDescription(
                device_key=DeviceKey(key="temperature", device_id=None),
                device_class=SensorDeviceClass.TEMPERATURE,
                native_unit_of_measurement=Units.TEMP_CELSIUS,
            )
        },
        entity_values={
            DeviceKey(key="temperature", device_id=None): SensorValue(
                device_key=DeviceKey(key="temperature", device_id=None),
                name="Temperature",
                native_value=30.3232,
            )
        },
        binary_entity_descriptions={},
        binary_entity_values={},
        events={},
    )


def test_with_precision_does_not_add_trailing_zeros():
    class MySensorData(SensorData):
        def _start_update(self, data: Any) -> None:
            self.set_precision(2)
            self.update_predefined_sensor(SensorLibrary.TEMPERATURE__CELSIUS, 30.9999)
            self.update_predefined_sensor(SensorLibrary.HUMIDITY__PERCENTAGE, 30)

    data = MySensorData()

    update = data.update(b"")
    assert update == SensorUpdate(
        title=None,
        devices={},
        entity_descriptions={
            DeviceKey(key="temperature", device_id=None): SensorDescription(
                device_key=DeviceKey(key="temperature", device_id=None),
                device_class=SensorDeviceClass.TEMPERATURE,
                native_unit_of_measurement=Units.TEMP_CELSIUS,
            ),
            DeviceKey(key="humidity", device_id=None): SensorDescription(
                device_key=DeviceKey(key="humidity", device_id=None),
                device_class=SensorDeviceClass.HUMIDITY,
                native_unit_of_measurement=Units.PERCENTAGE,
            ),
        },
        entity_values={
            DeviceKey(key="temperature", device_id=None): SensorValue(
                device_key=DeviceKey(key="temperature", device_id=None),
                name="Temperature",
                native_value=31.0,
            ),
            DeviceKey(key="humidity", device_id=None): SensorValue(
                device_key=DeviceKey(key="humidity", device_id=None),
                name="Humidity",
                native_value=30,
            ),
        },
        binary_entity_descriptions={},
        binary_entity_values={},
        events={},
    )


def test_event():
    update_count = 0

    class MySensorData(SensorData):
        def _start_update(self, data: Any) -> None:
            nonlocal update_count
            update_count += 1

            self.fire_event(
                key="dimmer",
                event_type=f"rotate_left_{update_count}",
                event_properties={"steps": 3},
                device_id="living_room",
            )

    data = MySensorData()

    update = data.update(b"")
    assert update == SensorUpdate(
        title=None,
        devices={},
        entity_descriptions={},
        entity_values={},
        binary_entity_descriptions={},
        binary_entity_values={},
        events={
            DeviceKey(key="dimmer", device_id="living_room"): Event(
                device_key=DeviceKey(key="dimmer", device_id="living_room"),
                name="Dimmer",
                event_type="rotate_left_1",
                event_properties={"steps": 3},
            )
        },
    )

    update = data.update(b"")
    assert update == SensorUpdate(
        title=None,
        devices={},
        entity_descriptions={},
        entity_values={},
        binary_entity_descriptions={},
        binary_entity_values={},
        events={
            DeviceKey(key="dimmer", device_id="living_room"): Event(
                device_key=DeviceKey(key="dimmer", device_id="living_room"),
                name="Dimmer",
                event_type="rotate_left_2",
                event_properties={"steps": 3},
            )
        },
    )



from __future__ import annotations

import dataclasses
from abc import abstractmethod
from datetime import date, datetime
from decimal import Decimal
from typing import Any

from .binary_sensor.device_class import BinarySensorDeviceClass
from .description import (
    BaseSensorDescription,
    BinarySensorDescription,
    SensorDescription,
)
from .device import DeviceKey
from .sensor.device_class import SensorDeviceClass
from .units import Units
from .value import BinarySensorValue, Event, SensorValue

_PRECISION_SENTINEL = -1


@dataclasses.dataclass(frozen=False)
class SensorDeviceInfo:
    name: str | None
    model: str | None
    manufacturer: str | None
    sw_version: str | None
    hw_version: str | None


@dataclasses.dataclass(frozen=True)
class SensorUpdate:
    title: str | None
    devices: dict[str | None, SensorDeviceInfo]
    entity_descriptions: dict[DeviceKey, SensorDescription] = dataclasses.field(
        default_factory=dict
    )
    entity_values: dict[DeviceKey, SensorValue] = dataclasses.field(
        default_factory=dict
    )
    binary_entity_descriptions: dict[DeviceKey, BinarySensorDescription] = (
        dataclasses.field(default_factory=dict)
    )
    binary_entity_values: dict[DeviceKey, BinarySensorValue] = dataclasses.field(
        default_factory=dict
    )
    events: dict[DeviceKey, Event] = dataclasses.field(default_factory=dict)


class SensorData:
    """Generate a sensor update."""

    def __init__(self) -> None:
        """Init sensor data."""
        self._title: str | None = None
        self._precision = _PRECISION_SENTINEL
        self._software_version: str | None = None
        self._device_id_info: dict[str | None, SensorDeviceInfo] = {}
        self._device_id_to_name: dict[str | None, str] = {}
        self._device_id_to_type: dict[str | None, str] = {}

        # Sensors
        self._sensor_descriptions: dict[DeviceKey, SensorDescription] = {}
        self._sensor_descriptions_updates: dict[DeviceKey, SensorDescription] = {}
        self._sensor_values: dict[DeviceKey, SensorValue] = {}
        self._sensor_values_updates: dict[DeviceKey, SensorValue] = {}

        # Binary Sensors
        self._binary_sensor_descriptions: dict[DeviceKey, BinarySensorDescription] = {}
        self._binary_sensor_descriptions_updates: dict[
            DeviceKey, BinarySensorDescription
        ] = {}
        self._binary_sensor_values: dict[DeviceKey, BinarySensorValue] = {}
        self._binary_sensor_values_updates: dict[DeviceKey, BinarySensorValue] = {}

        # Events
        self._events: dict[DeviceKey, Event] = {}
        self._events_updates: dict[DeviceKey, Event] = {}

    @property
    def descriptions(
        self,
    ) -> dict[DeviceKey, SensorDescription]:
        """Return the sensor data."""
        return self._sensor_descriptions

    @property
    def binary_descriptions(
        self,
    ) -> dict[DeviceKey, BinarySensorDescription]:
        """Return the binary sensor data."""
        return self._binary_sensor_descriptions

    @property
    def primary_device_id(self) -> str | None:
        """Return the primary device id."""
        if self._device_id_to_type:
            return list(self._device_id_to_type)[0]
        return None

    @property
    def precision(self) -> int:
        """Return the precision in decimal places."""
        return self._precision

    @property
    def title(self) -> str | None:
        """Return the title."""
        return self._title

    def set_title(self, title: str) -> None:
        """Set the title."""
        self._title = title

    def set_precision(self, precision: int) -> None:
        """Set the precision in decimal places.

        This is a convenience method to have native float values
        rounded to the expected number of decimal places.
        """
        if precision < 0:
            self._precision = _PRECISION_SENTINEL
        self._precision = precision

    def _get_device_info(self, device_id: str | None) -> SensorDeviceInfo:
        """Get device info."""
        if device_id not in self._device_id_info:
            self._device_id_info[device_id] = SensorDeviceInfo(
                None, None, None, None, None
            )
        return self._device_id_info[device_id]

    def set_device_manufacturer(
        self, manufacturer: str, device_id: str | None = None
    ) -> None:
        """Set the device manufacturer."""
        self._get_device_info(device_id).manufacturer = manufacturer

    def set_device_hw_version(
        self, hw_version: str, device_id: str | None = None
    ) -> None:
        """Set the device hardware version."""
        self._get_device_info(device_id).hw_version = hw_version

    def set_device_sw_version(
        self, sw_version: str, device_id: str | None = None
    ) -> None:
        """Set the device software version."""
        self._get_device_info(device_id).sw_version = sw_version

    def set_device_name(self, name: str, device_id: str | None = None) -> None:
        """Set the device name."""
        self._device_id_to_name[device_id] = name
        self._get_device_info(device_id).name = name

    def set_device_type(self, device_type: str, device_id: str | None = None) -> None:
        """Set the device type."""
        self._device_id_to_type[device_id] = device_type
        self._get_device_info(device_id).model = device_type

    @abstractmethod
    def _start_update(self, data: Any) -> None:
        """Update the data."""

    def supported(self, data: Any) -> bool:
        """Return True if the device is supported."""
        self._start_update(data)
        return bool(self._device_id_to_type)

    def update(self, data: Any) -> SensorUpdate:
        """Update a device."""
        # Ensure events from previous
        # updates are not carried over
        # as events are transient.
        self._events_updates.clear()
        self._start_update(data)
        return self._finish_update()

    def _finish_update(self) -> SensorUpdate:
        """Finish the update."""
        self._sensor_descriptions.update(self._sensor_descriptions_updates)
        self._sensor_values.update(self._sensor_values_updates)

        self._binary_sensor_descriptions.update(
            self._binary_sensor_descriptions_updates
        )
        self._binary_sensor_values.update(self._binary_sensor_values_updates)

        return SensorUpdate(
            title=self._title,
            devices=self._device_id_info,
            entity_descriptions=self._sensor_descriptions_updates,
            entity_values=self._sensor_values_updates,
            binary_entity_descriptions=self._binary_sensor_descriptions_updates,
            binary_entity_values=self._binary_sensor_values_updates,
            events=self._events_updates,
        )

    def update_predefined_binary_sensor(
        self,
        device_class: BinarySensorDeviceClass,
        native_value: bool | None,
        key: str | None = None,
        name: str | None = None,
        device_id: str | None = None,
    ) -> None:
        """Update a binary sensor by type."""
        assert device_class is not None  # nosec
        self.update_binary_sensor(
            key=key or device_class.value,
            name=name,
            native_value=native_value,
            device_class=device_class,
            device_id=device_id,
        )

    def update_predefined_sensor(
        self,
        base_description: BaseSensorDescription,
        native_value: None | str | int | float | date | datetime | Decimal,
        key: str | None = None,
        name: str | None = None,
        device_id: str | None = None,
    ) -> None:
        """Update a sensor by type."""
        assert base_description.device_class is not None  # nosec
        self.update_sensor(
            key=key or base_description.device_class.value,
            name=name,
            native_unit_of_measurement=base_description.native_unit_of_measurement,
            native_value=native_value,
            device_class=base_description.device_class,
            device_id=device_id,
        )

    def _get_key_name(self, key: str, device_id: str | None = None) -> str:
        """Set the device name."""
        return key.replace("_", " ").title()

    def get_device_name(self, device_id: str | None = None) -> str | None:
        """Get the device name."""
        return self._device_id_to_name.get(device_id) or self._device_id_to_type.get(
            device_id
        )

    def update_binary_sensor(
        self,
        key: str,
        native_value: bool | None,
        device_class: BinarySensorDeviceClass | None = None,
        name: str | None = None,
        device_id: str | None = None,
    ) -> None:
        """Update a sensor by type."""
        device_key = DeviceKey(key, device_id)
        self._binary_sensor_values_updates[device_key] = BinarySensorValue(
            name=name or self._get_key_name(key, device_id),
            device_key=device_key,
            native_value=native_value,
        )
        self._binary_sensor_descriptions_updates[device_key] = BinarySensorDescription(
            device_key=device_key,
            device_class=device_class,
        )

    def update_sensor(
        self,
        key: str,
        native_unit_of_measurement: Units | None,
        native_value: None | str | int | float | date | datetime | Decimal,
        device_class: SensorDeviceClass | None = None,
        name: str | None = None,
        device_id: str | None = None,
    ) -> None:
        """Update a sensor."""
        device_key = DeviceKey(key, device_id)
        if self.precision != _PRECISION_SENTINEL and isinstance(native_value, float):
            native_value = round(native_value, self.precision)
        self._sensor_values_updates[device_key] = SensorValue(
            name=name or self._get_key_name(key, device_id),
            device_key=device_key,
            native_value=native_value,
        )
        self._sensor_descriptions_updates[device_key] = SensorDescription(
            device_key=device_key,
            native_unit_of_measurement=native_unit_of_measurement,
            device_class=device_class,
        )

    def fire_event(
        self,
        key: str,
        event_type: str,
        event_properties: dict[str, str | int | float | None] | None = None,
        name: str | None = None,
        device_id: str | None = None,
    ) -> None:
        """Fire an event."""
        device_key = DeviceKey(key, device_id)
        self._events_updates[device_key] = Event(
            name=name or self._get_key_name(key, device_id),
            device_key=device_key,
            event_type=event_type,
            event_properties=event_properties,
        )



# Supported device classes

from ..base import BaseDeviceClass


class SensorDeviceClass(BaseDeviceClass):
    """Device class for sensors."""

    # acceleration (mG)
    ACCELERATION = "acceleration"

    # apparent power (VA)
    APPARENT_POWER = "apparent_power"

    # Air Quality Index
    AQI = "aqi"

    # % of battery that is left
    BATTERY = "battery"

    # ppm (parts per million) Carbon Monoxide gas concentration
    CO = "carbon_monoxide"

    # ppm (parts per million) Carbon Dioxide gas concentration
    CO2 = "carbon_dioxide"

    # conductivity (µS/cm)
    CONDUCTIVITY = "conductivity"

    # count (no unit)
    COUNT = "count"

    # current (A)
    CURRENT = "current"

    # date (ISO8601)
    DATE = "date"

    # dew point (°C, F)
    DEW_POINT = "dew_point"

    # distance (mm, m)
    DISTANCE = "distance"

    # fixed duration (TIME_DAYS, TIME_HOURS, TIME_MINUTES, TIME_SECONDS)
    DURATION = "duration"

    # energy (Wh, kWh, MWh)
    ENERGY = "energy"

    # Volume flow rate (m3 per hour)
    VOLUME_FLOW_RATE = "volume_flow_rate"

    # formaldehyde (µg/m³)
    FORMALDEHYDE = "formaldehyde"

    # frequency (Hz, kHz, MHz, GHz)
    FREQUENCY = "frequency"

    # gas (m³ or ft³)
    GAS = "gas"

    # gyroscope (° per second)
    GYROSCOPE = "gyroscope"

    # % of humidity in the air
    HUMIDITY = "humidity"

    # current light level (lx/lm)
    ILLUMINANCE = "illuminance"

    # impedance (Ohm)
    IMPEDANCE = "impedance"

    # keg size (L)
    KEG_SIZE = "keg_size"

    # keg type (no unit)
    KEG_TYPE = "keg_type"

    # % of moisture in the air or soil
    MOISTURE = "moisture"

    # mass (g, kg, lbs)
    MASS = "mass"

    # mass non-stabilized (kg/lbs)
    MASS_NON_STABILIZED = "mass_non_stabilized"

    # Amount of money (currency)
    MONETARY = "monetary"

    # Amount of NO2 (µg/m³)
    NITROGEN_DIOXIDE = "nitrogen_dioxide"

    # Amount of NO (µg/m³)
    NITROGEN_MONOXIDE = "nitrogen_monoxide"

    # Amount of N2O  (µg/m³)
    NITROUS_OXIDE = "nitrous_oxide"

    # Amount of O3 (µg/m³)
    OZONE = "ozone"

    # Packet id (no unit)
    PACKET_ID = "packet_id"

    # Particulate matter <= 0.1 μm (µg/m³)
    PM1 = "pm1"

    # Particulate matter <= 10 μm (µg/m³)
    PM10 = "pm10"

    # Particulate matter <= 2.5 μm (µg/m³)
    PM25 = "pm25"

    # Port count (no unit)
    PORT_COUNT = "port_count"

    # Port name (no unit)
    PORT_NAME = "port_name"

    # Port state (no unit)
    PORT_STATE = "port_state"

    # power factor (%)
    POWER_FACTOR = "power_factor"

    # power (W/kW)
    POWER = "power"

    # pressure (hPa/mbar)
    PRESSURE = "pressure"

    # rotation (°)
    ROTATION = "rotation"

    # reactive power (var)
    REACTIVE_POWER = "reactive_power"

    # signal strength (dB/dBm)
    SIGNAL_STRENGTH = "signal_strength"

    # specific gravity
    SPECIFIC_GRAVITY = "specific_gravity"

    # speed (m/s)
    SPEED = "speed"

    # Amount of SO2 (µg/m³)
    SULPHUR_DIOXIDE = "sulphur_dioxide"

    # temperature (°C/F)
    TEMPERATURE = "temperature"

    # time (s)
    TIME = "time"

    # timestamp (ISO8601)
    TIMESTAMP = "timestamp"

    # UV index (no unit)
    UV_INDEX = "uv_index"

    # Amount of VOC (µg/m³)
    VOLATILE_ORGANIC_COMPOUNDS = "volatile_organic_compounds"

    # voltage (V)
    VOLTAGE = "voltage"

    # Volume (ml, L)
    VOLUME = "volume"

    # Volume dispensed (L)
    VOLUME_DISPENSED = "volume_dispensed"

    # Volume start (L)
    VOLUME_START = "volume_start"

    # Water (L)
    WATER = "water"



from __future__ import annotations

from dataclasses import dataclass
from datetime import date, datetime
from decimal import Decimal

from .device import DeviceKey


@dataclass(frozen=True)
class BaseValue:
    device_key: DeviceKey
    name: str | None


@dataclass(frozen=True)
class SensorValue(BaseValue):
    """A class that describes sensor values."""

    native_value: str | int | float | date | datetime | Decimal | None


@dataclass(frozen=True)
class BinarySensorValue(BaseValue):
    """A class that describes sensor values."""

    native_value: bool | None


@dataclass(frozen=True)
class Event(BaseValue):
    """A class that describes device events."""

    event_type: str
    event_properties: dict[str, str | int | float | None] | None



__version__ = "2.18.1"

from .base import BaseDeviceClass
from .binary_sensor.device_class import BinarySensorDeviceClass
from .data import SensorData, SensorDeviceInfo, SensorUpdate
from .description import BinarySensorDescription, SensorDescription
from .device import DeviceKey
from .device_class import DeviceClass
from .library import SensorLibrary
from .sensor.device_class import SensorDeviceClass
from .units import Units
from .value import BaseValue, BinarySensorValue, Event, SensorValue

__all__ = [
    "BaseDeviceClass",
    "BaseValue",
    "BinarySensorValue",
    "BinarySensorDescription",
    "BinarySensorDeviceClass",
    "DeviceClass",
    "DeviceKey",
    "Event",
    "SensorData",
    "SensorDescription",
    "SensorDeviceClass",
    "SensorDeviceInfo",
    "SensorLibrary",
    "SensorUpdate",
    "SensorValue",
    "Units",
]



from __future__ import annotations

from dataclasses import dataclass

from .base import BaseDeviceClass
from .binary_sensor.device_class import BinarySensorDeviceClass
from .device import DeviceKey
from .sensor.device_class import SensorDeviceClass
from .units import Units


@dataclass(frozen=True)
class BaseDescription:
    """A base class for all descriptions."""

    device_class: BaseDeviceClass | None = None


@dataclass(frozen=True)
class BaseBinarySensorDescription(BaseDescription):
    device_class: BinarySensorDeviceClass | None = None


@dataclass(frozen=True)
class BaseSensorDescription(BaseDescription):
    device_class: SensorDeviceClass | None = None
    native_unit_of_measurement: Units | None = None


@dataclass(frozen=True)
class SensorDescriptionIds:
    device_key: DeviceKey


@dataclass(frozen=True)
class KeyedBaseDescription(BaseDescription, SensorDescriptionIds):
    """A class for sensors that use a key."""


@dataclass(frozen=True)
class SensorDescription(BaseSensorDescription, KeyedBaseDescription):
    """A class that describes sensors."""


@dataclass(frozen=True)
class BinarySensorDescription(BaseBinarySensorDescription, KeyedBaseDescription):
    """A class that describes binary sensors."""



from __future__ import annotations

import dataclasses


@dataclasses.dataclass(frozen=True)
class DeviceKey:
    """Key for a device.

    Example:
    device_id: outdoor_sensor_1
    key: temperature
    """

    key: str
    device_id: str | None = None



from __future__ import annotations

from .description import BaseSensorDescription
from .sensor.device_class import SensorDeviceClass
from .units import Units


class SensorLibrary:
    """A library of sensor common descriptions."""

    ACCELERATION__ACCELERATION_METERS_PER_SQUARE_SECOND = BaseSensorDescription(
        device_class=SensorDeviceClass.ACCELERATION,
        native_unit_of_measurement=Units.ACCELERATION_METERS_PER_SQUARE_SECOND,
    )
    BATTERY__PERCENTAGE = BaseSensorDescription(
        device_class=SensorDeviceClass.BATTERY,
        native_unit_of_measurement=Units.PERCENTAGE,
    )
    CONDUCTIVITY__CONDUCTIVITY = BaseSensorDescription(
        device_class=SensorDeviceClass.CONDUCTIVITY,
        native_unit_of_measurement=Units.CONDUCTIVITY,
    )
    COUNT__NONE = BaseSensorDescription(
        device_class=SensorDeviceClass.COUNT,
        native_unit_of_measurement=None,
    )
    CO2__CONCENTRATION_PARTS_PER_MILLION = BaseSensorDescription(
        device_class=SensorDeviceClass.CO2,
        native_unit_of_measurement=Units.CONCENTRATION_PARTS_PER_MILLION,
    )
    CURRENT__ELECTRIC_CURRENT_AMPERE = BaseSensorDescription(
        device_class=SensorDeviceClass.CURRENT,
        native_unit_of_measurement=Units.ELECTRIC_CURRENT_AMPERE,
    )
    DEW_POINT__TEMP_CELSIUS = BaseSensorDescription(
        device_class=SensorDeviceClass.DEW_POINT,
        native_unit_of_measurement=Units.TEMP_CELSIUS,
    )
    DISTANCE__LENGTH_METERS = BaseSensorDescription(
        device_class=SensorDeviceClass.DISTANCE,
        native_unit_of_measurement=Units.LENGTH_METERS,
    )
    DISTANCE__LENGTH_MILLIMETERS = BaseSensorDescription(
        device_class=SensorDeviceClass.DISTANCE,
        native_unit_of_measurement=Units.LENGTH_MILLIMETERS,
    )
    DURATION__TIME_SECONDS = BaseSensorDescription(
        device_class=SensorDeviceClass.DURATION,
        native_unit_of_measurement=Units.TIME_SECONDS,
    )
    ENERGY__ENERGY_KILO_WATT_HOUR = BaseSensorDescription(
        device_class=SensorDeviceClass.ENERGY,
        native_unit_of_measurement=Units.ENERGY_KILO_WATT_HOUR,
    )
    GYROSCOPE__GYROSCOPE_DEGREES_PER_SECOND = BaseSensorDescription(
        device_class=SensorDeviceClass.GYROSCOPE,
        native_unit_of_measurement=Units.GYROSCOPE_DEGREES_PER_SECOND,
    )
    FORMALDEHYDE__CONCENTRATION_MILLIGRAMS_PER_CUBIC_METER = BaseSensorDescription(
        device_class=SensorDeviceClass.FORMALDEHYDE,
        native_unit_of_measurement=Units.CONCENTRATION_MILLIGRAMS_PER_CUBIC_METER,
    )
    GAS__VOLUME_CUBIC_METERS = BaseSensorDescription(
        device_class=SensorDeviceClass.GAS,
        native_unit_of_measurement=Units.VOLUME_CUBIC_METERS,
    )
    HUMIDITY__PERCENTAGE = BaseSensorDescription(
        device_class=SensorDeviceClass.HUMIDITY,
        native_unit_of_measurement=Units.PERCENTAGE,
    )
    IMPEDANCE__OHM = BaseSensorDescription(
        device_class=SensorDeviceClass.IMPEDANCE,
        native_unit_of_measurement=Units.OHM,
    )
    KEG_SIZE__VOLUME_LITERS = BaseSensorDescription(
        device_class=SensorDeviceClass.KEG_SIZE,
        native_unit_of_measurement=Units.VOLUME_LITERS,
    )
    KEG_TYPE__NONE = BaseSensorDescription(
        device_class=SensorDeviceClass.KEG_TYPE,
        native_unit_of_measurement=None,
    )
    LIGHT__LIGHT_LUX = BaseSensorDescription(
        device_class=SensorDeviceClass.ILLUMINANCE,
        native_unit_of_measurement=Units.LIGHT_LUX,
    )
    MASS__MASS_KILOGRAMS = BaseSensorDescription(
        device_class=SensorDeviceClass.MASS,
        native_unit_of_measurement=Units.MASS_KILOGRAMS,
    )
    MASS__MASS_POUNDS = BaseSensorDescription(
        device_class=SensorDeviceClass.MASS,
        native_unit_of_measurement=Units.MASS_POUNDS,
    )
    MOISTURE__PERCENTAGE = BaseSensorDescription(
        device_class=SensorDeviceClass.MOISTURE,
        native_unit_of_measurement=Units.PERCENTAGE,
    )
    MASS_NON_STABILIZED__MASS_KILOGRAMS = BaseSensorDescription(
        device_class=SensorDeviceClass.MASS_NON_STABILIZED,
        native_unit_of_measurement=Units.MASS_KILOGRAMS,
    )
    PACKET_ID__NONE = BaseSensorDescription(
        device_class=SensorDeviceClass.PACKET_ID,
        native_unit_of_measurement=None,
    )
    PM1__CONCENTRATION_MICROGRAMS_PER_CUBIC_METER = BaseSensorDescription(
        device_class=SensorDeviceClass.PM1,
        native_unit_of_measurement=Units.CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
    )
    PM10__CONCENTRATION_MICROGRAMS_PER_CUBIC_METER = BaseSensorDescription(
        device_class=SensorDeviceClass.PM10,
        native_unit_of_measurement=Units.CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
    )
    PM25__CONCENTRATION_MICROGRAMS_PER_CUBIC_METER = BaseSensorDescription(
        device_class=SensorDeviceClass.PM25,
        native_unit_of_measurement=Units.CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
    )
    PORT_COUNT__NONE = BaseSensorDescription(
        device_class=SensorDeviceClass.PORT_COUNT,
        native_unit_of_measurement=None,
    )
    PORT_NAME__NONE = BaseSensorDescription(
        device_class=SensorDeviceClass.PORT_NAME,
        native_unit_of_measurement=None,
    )
    PORT_STATE__NONE = BaseSensorDescription(
        device_class=SensorDeviceClass.PORT_STATE,
        native_unit_of_measurement=None,
    )
    POWER__POWER_WATT = BaseSensorDescription(
        device_class=SensorDeviceClass.POWER,
        native_unit_of_measurement=Units.POWER_WATT,
    )
    PRESSURE__HPA = BaseSensorDescription(
        device_class=SensorDeviceClass.PRESSURE,
        native_unit_of_measurement=Units.PRESSURE_HPA,
    )
    PRESSURE__MBAR = BaseSensorDescription(
        device_class=SensorDeviceClass.PRESSURE,
        native_unit_of_measurement=Units.PRESSURE_MBAR,
    )
    ROTATION__DEGREE = BaseSensorDescription(
        device_class=SensorDeviceClass.ROTATION,
        native_unit_of_measurement=Units.DEGREE,
    )
    SPEED__SPEED_METERS_PER_SECOND = BaseSensorDescription(
        device_class=SensorDeviceClass.SPEED,
        native_unit_of_measurement=Units.SPEED_METERS_PER_SECOND,
    )
    TEMPERATURE__CELSIUS = BaseSensorDescription(
        device_class=SensorDeviceClass.TEMPERATURE,
        native_unit_of_measurement=Units.TEMP_CELSIUS,
    )
    TIMESTAMP__NONE = BaseSensorDescription(
        device_class=SensorDeviceClass.TIMESTAMP,
        native_unit_of_measurement=None,
    )
    UV_INDEX__NONE = BaseSensorDescription(
        device_class=SensorDeviceClass.UV_INDEX,
        native_unit_of_measurement=None,
    )
    VOLATILE_ORGANIC_COMPOUNDS__CONCENTRATION_MICROGRAMS_PER_CUBIC_METER = (
        BaseSensorDescription(
            device_class=SensorDeviceClass.VOLATILE_ORGANIC_COMPOUNDS,
            native_unit_of_measurement=Units.CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
        )
    )
    VOLTAGE__ELECTRIC_POTENTIAL_VOLT = BaseSensorDescription(
        device_class=SensorDeviceClass.VOLTAGE,
        native_unit_of_measurement=Units.ELECTRIC_POTENTIAL_VOLT,
    )
    VOLUME_DISPENSED__VOLUME_LITERS = BaseSensorDescription(
        device_class=SensorDeviceClass.VOLUME_DISPENSED,
        native_unit_of_measurement=Units.VOLUME_LITERS,
    )
    VOLUME_FLOW_RATE__VOLUME_FLOW_RATE_CUBIC_METERS_PER_HOUR = BaseSensorDescription(
        device_class=SensorDeviceClass.VOLUME_FLOW_RATE,
        native_unit_of_measurement=Units.VOLUME_FLOW_RATE_CUBIC_METERS_PER_HOUR,
    )
    VOLUME_START__VOLUME_LITERS = BaseSensorDescription(
        device_class=SensorDeviceClass.VOLUME_START,
        native_unit_of_measurement=Units.VOLUME_LITERS,
    )
    VOLUME__VOLUME_LITERS = BaseSensorDescription(
        device_class=SensorDeviceClass.VOLUME,
        native_unit_of_measurement=Units.VOLUME_LITERS,
    )
    VOLUME__VOLUME_MILLILITERS = BaseSensorDescription(
        device_class=SensorDeviceClass.VOLUME,
        native_unit_of_measurement=Units.VOLUME_MILLILITERS,
    )
    WATER__VOLUME_LITERS = BaseSensorDescription(
        device_class=SensorDeviceClass.WATER,
        native_unit_of_measurement=Units.VOLUME_LITERS,
    )



# Supported device classes

from .sensor.device_class import SensorDeviceClass  # backwards compatibility
from .sensor.device_class import SensorDeviceClass as DeviceClass

__all__ = ["DeviceClass", "SensorDeviceClass"]



from __future__ import annotations

from typing import Final

from .enum import StrEnum

# These are currently the same as Home Assistant sensors


class Units(StrEnum):
    # #### UNITS OF MEASUREMENT ####
    # Apparent power units
    POWER_VOLT_AMPERE: Final = "VA"

    # Power units
    POWER_WATT: Final = "W"
    POWER_KILO_WATT: Final = "kW"
    POWER_BTU_PER_HOUR: Final = "BTU/h"

    # Reactive power units
    POWER_VOLT_AMPERE_REACTIVE: Final = "var"

    # Energy units
    ENERGY_WATT_HOUR: Final = "Wh"
    ENERGY_KILO_WATT_HOUR: Final = "kWh"
    ENERGY_MEGA_WATT_HOUR: Final = "MWh"

    # Electric_current units
    ELECTRIC_CURRENT_MILLIAMPERE: Final = "mA"
    ELECTRIC_CURRENT_AMPERE: Final = "A"

    # Electric_potential units
    ELECTRIC_POTENTIAL_MILLIVOLT: Final = "mV"
    ELECTRIC_POTENTIAL_VOLT: Final = "V"

    # Degree units
    DEGREE: Final = "°"

    # Currency units
    CURRENCY_EURO: Final = "€"
    CURRENCY_DOLLAR: Final = "$"
    CURRENCY_CENT: Final = "¢"

    # Temperature units
    TEMP_CELSIUS: Final = "°C"
    TEMP_FAHRENHEIT: Final = "°F"
    TEMP_KELVIN: Final = "K"

    # Time units
    TIME_MICROSECONDS: Final = "μs"
    TIME_MILLISECONDS: Final = "ms"
    TIME_SECONDS: Final = "s"
    TIME_MINUTES: Final = "min"
    TIME_HOURS: Final = "h"
    TIME_DAYS: Final = "d"
    TIME_WEEKS: Final = "w"
    TIME_MONTHS: Final = "m"
    TIME_YEARS: Final = "y"

    # Length units
    LENGTH_MILLIMETERS: Final = "mm"
    LENGTH_CENTIMETERS: Final = "cm"
    LENGTH_METERS: Final = "m"
    LENGTH_KILOMETERS: Final = "km"

    LENGTH_INCHES: Final = "in"
    LENGTH_FEET: Final = "ft"
    LENGTH_YARD: Final = "yd"
    LENGTH_MILES: Final = "mi"

    # Frequency units
    FREQUENCY_HERTZ: Final = "Hz"
    FREQUENCY_KILOHERTZ: Final = "kHz"
    FREQUENCY_MEGAHERTZ: Final = "MHz"
    FREQUENCY_GIGAHERTZ: Final = "GHz"

    # Pressure units
    PRESSURE_PA: Final = "Pa"
    PRESSURE_HPA: Final = "hPa"
    PRESSURE_KPA: Final = "kPa"
    PRESSURE_BAR: Final = "bar"
    PRESSURE_CBAR: Final = "cbar"
    PRESSURE_MBAR: Final = "mbar"
    PRESSURE_MMHG: Final = "mmHg"
    PRESSURE_INHG: Final = "inHg"
    PRESSURE_PSI: Final = "psi"

    # Sound pressure units
    SOUND_PRESSURE_DB: Final = "dB"
    SOUND_PRESSURE_WEIGHTED_DBA: Final = "dBa"

    # Volume units
    VOLUME_LITERS: Final = "L"
    VOLUME_MILLILITERS: Final = "mL"
    VOLUME_CUBIC_METERS: Final = "m³"
    VOLUME_CUBIC_FEET: Final = "ft³"

    VOLUME_GALLONS: Final = "gal"
    VOLUME_FLUID_OUNCE: Final = "fl. oz."

    # Volume Flow Rate units
    VOLUME_FLOW_RATE_CUBIC_METERS_PER_HOUR: Final = "m³/h"
    VOLUME_FLOW_RATE_CUBIC_FEET_PER_MINUTE: Final = "ft³/m"

    # Area units
    AREA_SQUARE_METERS: Final = "m²"

    # Mass units
    MASS_GRAMS: Final = "g"
    MASS_KILOGRAMS: Final = "kg"
    MASS_MILLIGRAMS: Final = "mg"
    MASS_MICROGRAMS: Final = "µg"

    MASS_OUNCES: Final = "oz"
    MASS_POUNDS: Final = "lb"

    # Impedance units
    OHM: Final = "ohm"

    # Conductivity units
    CONDUCTIVITY: Final = "µS/cm"

    # Light units
    LIGHT_LUX: Final = "lx"

    # UV Index units
    UV_INDEX: Final = "UV index"

    # Percentage units
    PERCENTAGE: Final = "%"

    # Irradiation units
    IRRADIATION_WATTS_PER_SQUARE_METER: Final = "W/m²"
    IRRADIATION_BTUS_PER_HOUR_SQUARE_FOOT: Final = "BTU/(h×ft²)"

    # Precipitation units
    PRECIPITATION_MILLIMETERS_PER_HOUR: Final = "mm/h"
    PRECIPITATION_INCHES: Final = "in"
    PRECIPITATION_INCHES_PER_HOUR: Final = "in/h"

    # Concentration units
    CONCENTRATION_MICROGRAMS_PER_CUBIC_METER: Final = "µg/m³"
    CONCENTRATION_MILLIGRAMS_PER_CUBIC_METER: Final = "mg/m³"
    CONCENTRATION_MICROGRAMS_PER_CUBIC_FOOT: Final = "μg/ft³"
    CONCENTRATION_PARTS_PER_CUBIC_METER: Final = "p/m³"
    CONCENTRATION_PARTS_PER_MILLION: Final = "ppm"
    CONCENTRATION_PARTS_PER_BILLION: Final = "ppb"

    # Speed units
    SPEED_MILLIMETERS_PER_DAY: Final = "mm/d"
    SPEED_FEET_PER_SECOND: Final = "ft/s"
    SPEED_INCHES_PER_DAY: Final = "in/d"
    SPEED_METERS_PER_SECOND: Final = "m/s"
    SPEED_INCHES_PER_HOUR: Final = "in/h"
    SPEED_KILOMETERS_PER_HOUR: Final = "km/h"
    SPEED_KNOTS: Final = "kn"
    SPEED_MILES_PER_HOUR: Final = "mph"

    # Acceleration units
    ACCELERATION_METERS_PER_SQUARE_SECOND: Final = "m/s²"

    # Gyroscope units
    GYROSCOPE_DEGREES_PER_SECOND: Final = "°/s"

    # Signal_strength units
    SIGNAL_STRENGTH_DECIBELS: Final = "dB"
    SIGNAL_STRENGTH_DECIBELS_MILLIWATT: Final = "dBm"

    # Data units
    DATA_BITS: Final = "bit"
    DATA_KILOBITS: Final = "kbit"
    DATA_MEGABITS: Final = "Mbit"
    DATA_GIGABITS: Final = "Gbit"
    DATA_BYTES: Final = "B"
    DATA_KILOBYTES: Final = "kB"
    DATA_MEGABYTES: Final = "MB"
    DATA_GIGABYTES: Final = "GB"
    DATA_TERABYTES: Final = "TB"
    DATA_PETABYTES: Final = "PB"
    DATA_EXABYTES: Final = "EB"
    DATA_ZETTABYTES: Final = "ZB"
    DATA_YOTTABYTES: Final = "YB"
    DATA_KIBIBYTES: Final = "KiB"
    DATA_MEBIBYTES: Final = "MiB"
    DATA_GIBIBYTES: Final = "GiB"
    DATA_TEBIBYTES: Final = "TiB"
    DATA_PEBIBYTES: Final = "PiB"
    DATA_EXBIBYTES: Final = "EiB"
    DATA_ZEBIBYTES: Final = "ZiB"
    DATA_YOBIBYTES: Final = "YiB"

    # Data_rate units
    DATA_RATE_BITS_PER_SECOND: Final = "bit/s"
    DATA_RATE_KILOBITS_PER_SECOND: Final = "kbit/s"
    DATA_RATE_MEGABITS_PER_SECOND: Final = "Mbit/s"
    DATA_RATE_GIGABITS_PER_SECOND: Final = "Gbit/s"
    DATA_RATE_BYTES_PER_SECOND: Final = "B/s"
    DATA_RATE_KILOBYTES_PER_SECOND: Final = "kB/s"
    DATA_RATE_MEGABYTES_PER_SECOND: Final = "MB/s"
    DATA_RATE_GIGABYTES_PER_SECOND: Final = "GB/s"
    DATA_RATE_KIBIBYTES_PER_SECOND: Final = "KiB/s"
    DATA_RATE_MEBIBYTES_PER_SECOND: Final = "MiB/s"
    DATA_RATE_GIBIBYTES_PER_SECOND: Final = "GiB/s"

    # Specific gravity units
    SPECIFIC_GRAVITY = "SG"
